@@ -28,6 +28,166 @@ from warehouse.packaging.tables import (
 )
 
 
+def test_get_project_count(dbapp):
+    dbapp.engine.execute(packages.insert().values(name="foo1"))
+    dbapp.engine.execute(packages.insert().values(name="foo2"))
+    dbapp.engine.execute(packages.insert().values(name="foo3"))
+
+    assert dbapp.models.packaging.get_project_count() == 3
+
+
+def test_get_download_count(dbapp):
+    dbapp.engine.execute(packages.insert().values(name="foo"))
+    dbapp.engine.execute(releases.insert().values(name="foo", version="1.0"))
+    dbapp.engine.execute(release_files.insert().values(
+        name="foo",
+        version="1.0",
+        filename="foo-1.0.tar.gz",
+        downloads=15,
+    ))
+    dbapp.engine.execute(release_files.insert().values(
+        name="foo",
+        version="1.0",
+        filename="foo-1.0.tar.bz2",
+        downloads=12,
+    ))
+
+    assert dbapp.models.packaging.get_download_count() == 27
+
+
+def test_get_recently_updated(dbapp):
+    dbapp.engine.execute(packages.insert().values(name="foo1"))
+    dbapp.engine.execute(packages.insert().values(name="foo2"))
+    dbapp.engine.execute(packages.insert().values(name="foo3"))
+    dbapp.engine.execute(packages.insert().values(name="foo4"))
+    dbapp.engine.execute(packages.insert().values(name="foo5"))
+    dbapp.engine.execute(packages.insert().values(name="foo6"))
+    dbapp.engine.execute(packages.insert().values(name="foo7"))
+    dbapp.engine.execute(packages.insert().values(name="foo8"))
+    dbapp.engine.execute(packages.insert().values(name="foo9"))
+    dbapp.engine.execute(packages.insert().values(name="foo10"))
+    dbapp.engine.execute(packages.insert().values(name="foo11"))
+    dbapp.engine.execute(packages.insert().values(name="foo12"))
+
+    now = datetime.datetime.utcnow()
+
+    dbapp.engine.execute(releases.insert().values(
+        name="foo1", version="2.0", created=now,
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        name="foo1", version="1.0",
+        created=now - datetime.timedelta(seconds=5),
+    ))
+
+    dbapp.engine.execute(releases.insert().values(
+        name="foo2", version="1.0",
+        created=now - datetime.timedelta(seconds=10),
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        name="foo3", version="1.0",
+        created=now - datetime.timedelta(seconds=15),
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        name="foo4", version="1.0",
+        created=now - datetime.timedelta(seconds=20),
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        name="foo5", version="1.0",
+        created=now - datetime.timedelta(seconds=25),
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        name="foo6", version="1.0",
+        created=now - datetime.timedelta(seconds=30),
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        name="foo7", version="1.0",
+        created=now - datetime.timedelta(seconds=35),
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        name="foo8", version="1.0",
+        created=now - datetime.timedelta(seconds=40),
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        name="foo9", version="1.0",
+        created=now - datetime.timedelta(seconds=45),
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        name="foo10", version="1.0",
+        created=now - datetime.timedelta(seconds=50),
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        name="foo11", version="1.0",
+        created=now - datetime.timedelta(seconds=55),
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        name="foo12", version="1.0",
+        created=now - datetime.timedelta(seconds=60),
+    ))
+
+    assert dbapp.models.packaging.get_recently_updated() == [
+        {
+            "name": "foo1",
+            "version": "2.0",
+            "summary": None,
+            "created": now,
+        },
+        {
+            "name": "foo2",
+            "version": "1.0",
+            "summary": None,
+            "created": now - datetime.timedelta(seconds=10),
+        },
+        {
+            "name": "foo3",
+            "version": "1.0",
+            "summary": None,
+            "created": now - datetime.timedelta(seconds=15),
+        },
+        {
+            "name": "foo4",
+            "version": "1.0",
+            "summary": None,
+            "created": now - datetime.timedelta(seconds=20),
+        },
+        {
+            "name": "foo5",
+            "version": "1.0",
+            "summary": None,
+            "created": now - datetime.timedelta(seconds=25),
+        },
+        {
+            "name": "foo6",
+            "version": "1.0",
+            "summary": None,
+            "created": now - datetime.timedelta(seconds=30),
+        },
+        {
+            "name": "foo7",
+            "version": "1.0",
+            "summary": None,
+            "created": now - datetime.timedelta(seconds=35),
+        },
+        {
+            "name": "foo8",
+            "version": "1.0",
+            "summary": None,
+            "created": now - datetime.timedelta(seconds=40),
+        },
+        {
+            "name": "foo9",
+            "version": "1.0",
+            "summary": None,
+            "created": now - datetime.timedelta(seconds=45),
+        },
+        {
+            "name": "foo10",
+            "version": "1.0",
+            "summary": None,
+            "created": now - datetime.timedelta(seconds=50),
+        },
+    ]
+
+
 @pytest.mark.parametrize("projects", [
     ["foo", "bar", "zap"],
     ["fail", "win", "YeS"],
@@ -321,27 +481,214 @@ def test_get_project_versions(dbapp):
     ]
 
 
-def test_get_release(dbapp, monkeypatch):
-    release = pretend.stub()
-    get_releases = pretend.call_recorder(lambda p, v: [release])
+def test_get_release(dbapp):
+    created = datetime.datetime.utcnow()
 
-    monkeypatch.setattr(dbapp.models.packaging, "_get_releases", get_releases)
+    dbapp.engine.execute(packages.insert().values(name="test-project"))
+    dbapp.engine.execute(releases.insert().values(
+        created=created,
+        name="test-project",
+        version="1.0",
+        author="John Doe",
+        author_email="john.doe@example.com",
+        maintainer="Jane Doe",
+        maintainer_email="jane.doe@example.com",
+        home_page="https://example.com/",
+        license="Apache License v2.0",
+        summary="A Test Project",
+        description="A Longer Test Project",
+        keywords="foo,bar,wat",
+        platform="All",
+        download_url="https://example.com/downloads/test-project-1.0.tar.gz",
+        _pypi_ordering=1,
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        created=created,
+        name="test-project",
+        version="2.0",
+        author="John Doe",
+        author_email="john.doe@example.com",
+        maintainer="Jane Doe",
+        maintainer_email="jane.doe@example.com",
+        home_page="https://example.com/",
+        license="Apache License v2.0",
+        summary="A Test Project",
+        description="A Longer Test Project",
+        keywords="foo,bar,wat",
+        platform="All",
+        download_url="https://example.com/downloads/test-project-1.0.tar.gz",
+        _pypi_ordering=2,
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="1.0",
+        kind=4,
+        specifier="requests (>=2.0)",
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="2.0",
+        kind=4,
+        specifier="requests (>=2.0)",
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="1.0",
+        kind=5,
+        specifier="test-project-old",
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="2.0",
+        kind=5,
+        specifier="test-project-old",
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="1.0",
+        kind=8,
+        specifier="Repository,git://git.example.com/",
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="2.0",
+        kind=8,
+        specifier="Repository,git://git.example.com/",
+    ))
 
-    assert dbapp.models.packaging.get_release("test-project", "1.0") is release
-    assert get_releases.calls == [pretend.call("test-project", "1.0")]
+    test_release = dbapp.models.packaging.get_release("test-project", "1.0")
+
+    assert test_release == {
+        "name": "test-project",
+        "version": "1.0",
+        "author": "John Doe",
+        "author_email": "john.doe@example.com",
+        "maintainer": "Jane Doe",
+        "maintainer_email": "jane.doe@example.com",
+        "home_page": "https://example.com/",
+        "license": "Apache License v2.0",
+        "summary": "A Test Project",
+        "description": "A Longer Test Project",
+        "keywords": "foo,bar,wat",
+        "platform": "All",
+        "download_url": ("https://example.com/downloads/"
+                         "test-project-1.0.tar.gz"),
+        "requires_dist": ["requests (>=2.0)"],
+        "provides_dist": ["test-project-old"],
+        "project_url": {"Repository": "git://git.example.com/"},
+        "created": created,
+    }
 
 
-def test_get_releases(dbapp, monkeypatch):
-    release = pretend.stub()
-    get_releases = pretend.call_recorder(lambda p: [release])
+def test_get_releases(dbapp):
+    created = datetime.datetime.utcnow()
 
-    monkeypatch.setattr(dbapp.models.packaging, "_get_releases", get_releases)
+    dbapp.engine.execute(packages.insert().values(name="test-project"))
+    dbapp.engine.execute(releases.insert().values(
+        created=created,
+        name="test-project",
+        version="1.0",
+        author="John Doe",
+        author_email="john.doe@example.com",
+        maintainer="Jane Doe",
+        maintainer_email="jane.doe@example.com",
+        home_page="https://example.com/",
+        license="Apache License v2.0",
+        summary="A Test Project",
+        description="A Longer Test Project",
+        keywords="foo,bar,wat",
+        platform="All",
+        download_url="https://example.com/downloads/test-project-1.0.tar.gz",
+        _pypi_ordering=1,
+    ))
+    dbapp.engine.execute(releases.insert().values(
+        created=created,
+        name="test-project",
+        version="2.0",
+        author="John Doe",
+        author_email="john.doe@example.com",
+        maintainer="Jane Doe",
+        maintainer_email="jane.doe@example.com",
+        home_page="https://example.com/",
+        license="Apache License v2.0",
+        summary="A Test Project",
+        description="A Longer Test Project",
+        keywords="foo,bar,wat",
+        platform="All",
+        download_url="https://example.com/downloads/test-project-1.0.tar.gz",
+        _pypi_ordering=2,
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="1.0",
+        kind=4,
+        specifier="requests (>=2.0)",
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="2.0",
+        kind=4,
+        specifier="requests (>=2.0)",
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="1.0",
+        kind=5,
+        specifier="test-project-old",
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="2.0",
+        kind=5,
+        specifier="test-project-old",
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="1.0",
+        kind=8,
+        specifier="Repository,git://git.example.com/",
+    ))
+    dbapp.engine.execute(release_dependencies.insert().values(
+        name="test-project",
+        version="2.0",
+        kind=8,
+        specifier="Repository,git://git.example.com/",
+    ))
 
-    releases = dbapp.models.packaging.get_releases("test-project")
-
-    assert releases == [release]
-    assert releases[0] is release
-    assert get_releases.calls == [pretend.call("test-project")]
+    assert dbapp.models.packaging.get_releases("test-project") == [
+        {
+            "name": "test-project",
+            "version": "2.0",
+            "author": "John Doe",
+            "author_email": "john.doe@example.com",
+            "maintainer": "Jane Doe",
+            "maintainer_email": "jane.doe@example.com",
+            "home_page": "https://example.com/",
+            "license": "Apache License v2.0",
+            "summary": "A Test Project",
+            "keywords": "foo,bar,wat",
+            "platform": "All",
+            "download_url": ("https://example.com/downloads/"
+                             "test-project-1.0.tar.gz"),
+            "created": created,
+        },
+        {
+            "name": "test-project",
+            "version": "1.0",
+            "author": "John Doe",
+            "author_email": "john.doe@example.com",
+            "maintainer": "Jane Doe",
+            "maintainer_email": "jane.doe@example.com",
+            "home_page": "https://example.com/",
+            "license": "Apache License v2.0",
+            "summary": "A Test Project",
+            "keywords": "foo,bar,wat",
+            "platform": "All",
+            "download_url": ("https://example.com/downloads/"
+                             "test-project-1.0.tar.gz"),
+            "created": created,
+        },
+    ]
 
 
 @pytest.mark.parametrize("exists", [True, False])
@@ -509,129 +856,3 @@ def test_get_download_counts(dbapp):
 
     assert counts == {"last_day": 30, "last_week": 30, "last_month": 30}
     assert len(mget.calls) == 3
-
-
-@pytest.mark.parametrize("version", [True, False])
-def test_real_get_releases_multiple(version, dbapp):
-    dbapp.engine.execute(packages.insert().values(name="test-project"))
-    dbapp.engine.execute(releases.insert().values(
-        name="test-project",
-        version="1.0",
-        author="John Doe",
-        author_email="john.doe@example.com",
-        maintainer="Jane Doe",
-        maintainer_email="jane.doe@example.com",
-        home_page="https://example.com/",
-        license="Apache License v2.0",
-        summary="A Test Project",
-        description="A Longer Test Project",
-        keywords="foo,bar,wat",
-        platform="All",
-        download_url="https://example.com/downloads/test-project-1.0.tar.gz",
-        _pypi_ordering=1,
-    ))
-    dbapp.engine.execute(releases.insert().values(
-        name="test-project",
-        version="2.0",
-        author="John Doe",
-        author_email="john.doe@example.com",
-        maintainer="Jane Doe",
-        maintainer_email="jane.doe@example.com",
-        home_page="https://example.com/",
-        license="Apache License v2.0",
-        summary="A Test Project",
-        description="A Longer Test Project",
-        keywords="foo,bar,wat",
-        platform="All",
-        download_url="https://example.com/downloads/test-project-1.0.tar.gz",
-        _pypi_ordering=2,
-    ))
-    dbapp.engine.execute(release_dependencies.insert().values(
-        name="test-project",
-        version="1.0",
-        kind=4,
-        specifier="requests (>=2.0)",
-    ))
-    dbapp.engine.execute(release_dependencies.insert().values(
-        name="test-project",
-        version="2.0",
-        kind=4,
-        specifier="requests (>=2.0)",
-    ))
-    dbapp.engine.execute(release_dependencies.insert().values(
-        name="test-project",
-        version="1.0",
-        kind=5,
-        specifier="test-project-old",
-    ))
-    dbapp.engine.execute(release_dependencies.insert().values(
-        name="test-project",
-        version="2.0",
-        kind=5,
-        specifier="test-project-old",
-    ))
-    dbapp.engine.execute(release_dependencies.insert().values(
-        name="test-project",
-        version="1.0",
-        kind=8,
-        specifier="Repository,git://git.example.com/",
-    ))
-    dbapp.engine.execute(release_dependencies.insert().values(
-        name="test-project",
-        version="2.0",
-        kind=8,
-        specifier="Repository,git://git.example.com/",
-    ))
-
-    test_releases = dbapp.models.packaging._get_releases(
-        "test-project",
-        "1.0" if version else None,
-    )
-
-    expected = [
-        {
-            "name": "test-project",
-            "version": "2.0",
-            "author": "John Doe",
-            "author_email": "john.doe@example.com",
-            "maintainer": "Jane Doe",
-            "maintainer_email": "jane.doe@example.com",
-            "home_page": "https://example.com/",
-            "license": "Apache License v2.0",
-            "summary": "A Test Project",
-            "description": "A Longer Test Project",
-            "keywords": "foo,bar,wat",
-            "platform": "All",
-            "download_url": ("https://example.com/downloads/"
-                             "test-project-1.0.tar.gz"),
-            "requires_dist": ["requests (>=2.0)"],
-            "provides_dist": ["test-project-old"],
-            "project_url": {"Repository": "git://git.example.com/"},
-            "created": None,
-        },
-        {
-            "name": "test-project",
-            "version": "1.0",
-            "author": "John Doe",
-            "author_email": "john.doe@example.com",
-            "maintainer": "Jane Doe",
-            "maintainer_email": "jane.doe@example.com",
-            "home_page": "https://example.com/",
-            "license": "Apache License v2.0",
-            "summary": "A Test Project",
-            "description": "A Longer Test Project",
-            "keywords": "foo,bar,wat",
-            "platform": "All",
-            "download_url": ("https://example.com/downloads/"
-                             "test-project-1.0.tar.gz"),
-            "requires_dist": ["requests (>=2.0)"],
-            "provides_dist": ["test-project-old"],
-            "project_url": {"Repository": "git://git.example.com/"},
-            "created": None,
-        },
-    ]
-
-    if version:
-        assert test_releases == [expected[1]]
-    else:
-        assert test_releases == expected
